@@ -163,7 +163,7 @@ def plot_dielectric_modulus(omega, Z, c0, Z_comp=None,
 
 
 def plot_dielectric_properties(omega, Z, c0, Z_comp=None, title="", show=True, save=False, logscale="permittivity",
-                               labels=None, append=False, markers=[None, None], **plotkwargs):
+                               labels=None, append=False, markers=[None, None], legend=True, limits=None, **plotkwargs):
     '''
     Parameters
     ----------
@@ -190,6 +190,8 @@ def plot_dielectric_properties(omega, Z, c0, Z_comp=None, title="", show=True, s
         Give custom labels. Needs to be a list of length 2.
     append: bool, optional
         Decide if you want to show plot or add line to existing plot.
+    legend: bool, optional
+        Switch legend on/off
 
     '''
     eps_r, cond_fit = return_diel_properties(omega, Z, c0)
@@ -212,13 +214,15 @@ def plot_dielectric_properties(omega, Z, c0, Z_comp=None, title="", show=True, s
     plt.title("Relative permittivity")
     plt.ylabel("Relative permittivity")
     plt.xlabel('Frequency / Hz')
+    if limits:
+        plt.ylim(limits[0])
     if logscale == 'permittivity' or logscale == 'both':
         plt.yscale('log')
     plt.xscale('log')
     plt.plot(omega / (2. * np.pi), eps_r, label=labels[0], marker=markers[0], **plotkwargs)
     if Z_comp is not None:
-        plt.plot(omega / (2. * np.pi), eps_r2, label=labels[1], marker=markers[1], **plotkwargs)
-    if Z_comp is not None or append is True:
+        plt.plot(omega / (2. * np.pi), eps_r2, linestyle='--', label=labels[1], marker=markers[1], **plotkwargs)
+    if legend:
         plt.legend()
 
     if len(axes) < 2:
@@ -228,14 +232,16 @@ def plot_dielectric_properties(omega, Z, c0, Z_comp=None, title="", show=True, s
 
     plt.title("Conductivity")
     plt.ylabel(r"Conductivity / S$\cdot$m$^{-1}$")
+    if limits:
+        plt.ylim(limits[1])
     plt.xlabel('Frequency / Hz')
     if logscale == 'conductivity' or logscale == 'both':
         plt.yscale('log')
     plt.xscale('log')
     plt.plot(omega / (2. * np.pi), cond_fit, label=labels[0], marker=markers[0], **plotkwargs)
     if Z_comp is not None:
-        plt.plot(omega / (2. * np.pi), cond_fit2, label=labels[1], marker=markers[1], **plotkwargs)
-    if Z_comp is not None or append is True:
+        plt.plot(omega / (2. * np.pi), cond_fit2, linestyle='--', label=labels[1], marker=markers[1], **plotkwargs)
+    if legend:
         plt.legend()
     plt.tight_layout()
     if save and not append:
@@ -246,8 +252,102 @@ def plot_dielectric_properties(omega, Z, c0, Z_comp=None, title="", show=True, s
         plt.close()
 
 
-def plot_cole_cole(omega, Z, c0, Z_comp=None, diff=False,
-                   title="", show=True, save=False, labels=None):
+def plot_comparison_dielectric_properties(omega, Z, c0, Z_comp,
+                                          title="", show=True, save=False, residual="relative",
+                                          label=None, append=False, marker=None,
+                                          legend=True, limits=None, **plotkwargs):
+    '''
+    Parameters
+    ----------
+
+    omega: :class:`numpy.ndarray`, double
+        frequency array
+    Z: :class:`numpy.ndarray`, complex
+        impedance array
+    c0: double
+        unit capacitance of device
+    Z_comp: :class:`numpy.ndarray`, complex
+        complex-valued impedance array to compare the properties of two data sets.
+    title: str, optional
+        title of plot. Default is an empty string.
+    show: bool, optional
+        show figure (default is True)
+    save: bool, optional
+        save figure to pdf (default is False). Name of figure starts with `title`
+        and ends with `_dielectric_properties.pdf`.
+    logscale: str, optional
+        Decide what you want to plot using log scale.
+        Possible are `permittivity`, `conductivity` and `both`
+    label: str, optional
+        Give custom label. Needs to be a str.
+    append: bool, optional
+        Decide if you want to show plot or add line to existing plot.
+    legend: bool, optional
+        Switch legend on/off
+
+    '''
+    eps_r, cond_fit = return_diel_properties(omega, Z, c0)
+    eps_r2, cond_fit2 = return_diel_properties(omega, Z_comp, c0)
+
+    if residual == "relative":
+        diff_eps = 100. * np.abs((eps_r - eps_r2) / eps_r2)
+        diff_sigma = 100. * np.abs((cond_fit - cond_fit2) / cond_fit2)
+        ylabel = 'Relative difference / %'
+    elif residual == "absolute":
+        diff_eps = eps_r - eps_r2
+        diff_sigma = cond_fit - cond_fit2
+        ylabel = r"Absolute Difference"
+    else:
+        raise RuntimeError("""residual must be either `relative` or `absolute`,
+                              but not {}""".format(residual))
+
+    axes = []
+    plt.figure("comparisondielectricproperties")
+    axes = plt.gcf().axes
+
+    if len(axes) < 2:
+        plt.suptitle(title, y=1.05)
+        plt.subplot(211)
+    else:
+        plt.sca(axes[0])
+
+    plt.title("Relative permittivity")
+    plt.ylabel(ylabel)
+    plt.xlabel('Frequency / Hz')
+    if limits:
+        plt.ylim(limits[0])
+    plt.xscale('log')
+    plt.plot(omega / (2. * np.pi), diff_eps, label=label, marker=marker, **plotkwargs)
+    if legend:
+        plt.legend()
+
+    if len(axes) < 2:
+        plt.subplot(212)
+    else:
+        plt.sca(axes[1])
+
+    plt.title("Conductivity")
+    if residual == "absolute":
+        ylabel += r" / S$\cdot$m$^{-1}$"
+    plt.ylabel(ylabel)
+    if limits:
+        plt.ylim(limits[1])
+    plt.xlabel('Frequency / Hz')
+    plt.xscale('log')
+    plt.plot(omega / (2. * np.pi), diff_sigma, label=label, marker=marker, **plotkwargs)
+    if legend:
+        plt.legend()
+    plt.tight_layout()
+    if save and not append:
+        plt.savefig(str(title).replace(" ", "_") + "_comparison_{}_dielectric_properties.pdf".format(residual))
+    if show and not append:
+        plt.show()
+    elif not show and not append:
+        plt.close()
+
+
+def plot_cole_cole(omega, Z, c0, Z_comp=None, append=False, legend=True, markers=[None, None],
+                   title="", show=True, save=False, labels=None, limits=None):
     '''
     Parameters
     ----------
@@ -262,8 +362,6 @@ def plot_cole_cole(omega, Z, c0, Z_comp=None, diff=False,
         complex-valued impedance array. Might be used to compare the properties of two data sets.
     title: str, optional
         title of plot. Default is an empty string.
-    diff: bool, optional
-        Compare two results numerically by plotting difference (default False)
     show: bool, optional
         show figure (default is True)
     save: bool, optional
@@ -275,6 +373,12 @@ def plot_cole_cole(omega, Z, c0, Z_comp=None, diff=False,
     labels: list, optional
         Give custom labels. Needs to be a list of length 2.
     '''
+    axes = []
+    plt.figure("colecoleplot")
+    axes = plt.gcf().axes
+    if len(axes) == 1:
+        plt.sca(axes[0])
+
     eps_r, cond_fit = return_diel_properties(omega, Z, c0)
     epsc_fit = eps_r - 1j * cond_fit / (e0 * omega)
     if labels is None:
@@ -283,35 +387,26 @@ def plot_cole_cole(omega, Z, c0, Z_comp=None, diff=False,
     if Z_comp is not None:
         eps_r2, cond_fit2 = return_diel_properties(omega, Z_comp, c0)
         epsc_fit2 = eps_r2 - 1j * cond_fit2 / (e0 * omega)
-        if diff:
-            plt.figure()
-            plt.subplot(211)
 
     plt.title("Cole-Cole plot")
     plt.xlabel(r"Re $\varepsilon$")
     plt.ylabel(r"-Im $\varepsilon$")
-    plt.plot(epsc_fit.real, -epsc_fit.imag, label=labels[0])
+    plt.plot(epsc_fit.real, -epsc_fit.imag, label=labels[0], marker=markers[0])
+    if limits is not None:
+        plt.xlim(limits[0])
+        plt.ylim(limits[1])
     if Z_comp is not None:
-        plt.plot(epsc_fit2.real, -epsc_fit2.imag, label=labels[1])
-        plt.legend()
-
-    if Z_comp is not None and diff:
-        plt.subplot(212)
-        plt.title("Comparison")
-        plt.ylabel("Relative difference / %")
-        plt.xlabel('Frequency / Hz')
-        plt.xscale('log')
-        plt.plot(omega / (2. * np.pi), 100. * np.abs((epsc_fit.real - epsc_fit2.real) / epsc_fit.real), label="real")
-        plt.plot(omega / (2. * np.pi), 100. * np.abs((epsc_fit.imag - epsc_fit2.imag) / epsc_fit.imag), label="imag")
-        plt.plot(omega / (2. * np.pi), 100. * np.abs((epsc_fit - epsc_fit2) / epsc_fit), label="abs")
+        plt.plot(epsc_fit2.real, -epsc_fit2.imag, label=labels[1], marker=markers[1])
+    if legend:
         plt.legend()
     plt.tight_layout()
-    if save:
+    if save and not append:
         plt.savefig(str(title).replace(" ", "_") + "_cole_cole_plot.pdf")
-    if show:
+    if show and not append:
         plt.show()
-    else:
+    elif not show and not append:
         plt.close()
+    return
 
 
 def plot_bode(omega, Z, title="", Z_fit=None, show=True, save=False, Z_comp=None,
@@ -488,7 +583,7 @@ def plot_resistance_capacitance(omega, Z, title="", Z_fit=None, show=True, save=
 def plot_impedance(omega, Z, title="", Z_fit=None, show=True, save=False, Z_comp=None,
                    labels=["Data", "Best fit", "Init fit"], residual="parts", sign=False,
                    Zlog=False, append=False, limits_residual=None,
-                   omega_fit=None, omega_comp=None, legend=True, compare=True):
+                   omega_fit=None, omega_comp=None, legend=True, compare=True, **plotkwargs):
     """Plot the `result` and compare it to data `Z`.
 
     Generates 4 subplots showing the real and imaginary parts over
@@ -562,11 +657,11 @@ def plot_impedance(omega, Z, title="", Z_fit=None, show=True, save=False, Z_comp
     plt.title("Impedance real part")
     plt.ylabel(r"Re Z / $\Omega$")
     plt.xlabel('Frequency / Hz')
-    plt.plot(omega / (2. * np.pi), Z.real, label=labels[0])
+    plt.plot(omega / (2. * np.pi), Z.real, label=labels[0], **plotkwargs)
     if Z_fit is not None:
-        plt.plot(omega_fit / (2. * np.pi), Z_fit.real, linestyle='--', label=labels[1], lw=3)
+        plt.plot(omega_fit / (2. * np.pi), Z_fit.real, linestyle='--', label=labels[1], **plotkwargs)
     if Z_comp is not None:
-        plt.plot(omega_comp / (2. * np.pi), Z_comp.real, linestyle='-.', label=labels[2], lw=3)
+        plt.plot(omega_comp / (2. * np.pi), Z_comp.real, linestyle='-.', label=labels[2], **plotkwargs)
     if legend:
         plt.legend()
     # plot imaginary part of impedance
@@ -582,41 +677,41 @@ def plot_impedance(omega, Z, title="", Z_fit=None, show=True, save=False, Z_comp
         plt.yscale('log')
         if np.all(np.less_equal(Z.imag, 0)):
             plt.ylabel(r"-Im Z / $\Omega$")
-            plt.plot(omega / (2. * np.pi), -Z.imag, label=labels[0])
+            plt.plot(omega / (2. * np.pi), -Z.imag, label=labels[0], **plotkwargs)
             if Z_fit is not None:
-                plt.plot(omega_fit / (2. * np.pi), -Z_fit.imag, '--', label=labels[1])
+                plt.plot(omega_fit / (2. * np.pi), -Z_fit.imag, '--', label=labels[1], **plotkwargs)
             if Z_comp is not None:
-                plt.plot(omega_comp / (2. * np.pi), -Z_comp.imag, '-.', label=labels[2])
+                plt.plot(omega_comp / (2. * np.pi), -Z_comp.imag, '-.', label=labels[2], **plotkwargs)
 
         elif np.all(np.greater_equal(Z.imag, 0)):
-            plt.plot(omega / (2. * np.pi), Z.imag, label=labels[0])
+            plt.plot(omega / (2. * np.pi), Z.imag, label=labels[0], **plotkwargs)
             if Z_fit is not None:
-                plt.plot(omega_fit / (2. * np.pi), Z_fit.imag, '--', label=labels[1])
+                plt.plot(omega_fit / (2. * np.pi), Z_fit.imag, '--', label=labels[1], **plotkwargs)
             if Z_comp is not None:
-                plt.plot(omega_comp / (2. * np.pi), Z_comp.imag, '-.', label=labels[2])
+                plt.plot(omega_comp / (2. * np.pi), Z_comp.imag, '-.', label=labels[2], **plotkwargs)
 
         elif np.where(Z.imag < 0)[0].size > np.where(Z.imag > 0)[0].size:
             plt.ylabel(r"-Im Z / $\Omega$")
-            plt.plot(omega / (2. * np.pi), -Z.imag, label=labels[0])
+            plt.plot(omega / (2. * np.pi), -Z.imag, label=labels[0], **plotkwargs)
             if Z_fit is not None:
-                plt.plot(omega_fit / (2. * np.pi), -Z_fit.imag, '--', label=labels[1])
+                plt.plot(omega_fit / (2. * np.pi), -Z_fit.imag, '--', label=labels[1], **plotkwargs)
             if Z_comp is not None:
-                plt.plot(omega_comp / (2. * np.pi), -Z_comp.imag, '-.', label=labels[2])
+                plt.plot(omega_comp / (2. * np.pi), -Z_comp.imag, '-.', label=labels[2], **plotkwargs)
 
         else:
-            plt.plot(omega / (2. * np.pi), Z.imag, label=labels[0])
+            plt.plot(omega / (2. * np.pi), Z.imag, label=labels[0], **plotkwargs)
             if Z_fit is not None:
-                plt.plot(omega_fit / (2. * np.pi), Z_fit.imag, '--', label=labels[1])
+                plt.plot(omega_fit / (2. * np.pi), Z_fit.imag, '--', label=labels[1], **plotkwargs)
             if Z_comp is not None:
-                plt.plot(omega_comp / (2. * np.pi), Z_comp.imag, '-.', label=labels[2])
+                plt.plot(omega_comp / (2. * np.pi), Z_comp.imag, '-.', label=labels[2], **plotkwargs)
 
     else:
-        plt.plot(omega / (2. * np.pi), Z.imag, label=labels[0])
+        plt.plot(omega / (2. * np.pi), Z.imag, label=labels[0], **plotkwargs)
 
         if Z_fit is not None:
-            plt.plot(omega_fit / (2. * np.pi), Z_fit.imag, '--', label=labels[1], lw=3)
+            plt.plot(omega_fit / (2. * np.pi), Z_fit.imag, '--', label=labels[1], **plotkwargs)
         if Z_comp is not None:
-            plt.plot(omega_comp / (2. * np.pi), Z_comp.imag, '-.', label=labels[2], lw=3)
+            plt.plot(omega_comp / (2. * np.pi), Z_comp.imag, '-.', label=labels[2], **plotkwargs)
     if legend:
         plt.legend()
     # plot real vs negative imaginary part
@@ -630,19 +725,20 @@ def plot_impedance(omega, Z, title="", Z_fit=None, show=True, save=False, Z_comp
     if Zlog:
         plt.xscale('log')
         plt.yscale('log')
-    plt.plot(Z.real, -Z.imag, 'o', label=labels[0])
+    plt.plot(Z.real, -Z.imag, 'o', label=labels[0], **plotkwargs)
     if Z_fit is not None:
-        plt.plot(Z_fit.real, -Z_fit.imag, '^', label=labels[1])
+        plt.plot(Z_fit.real, -Z_fit.imag, '^', label=labels[1], **plotkwargs)
     if Z_comp is not None:
-        plt.plot(Z_comp.real, -Z_comp.imag, 'v', label=labels[2])
+        plt.plot(Z_comp.real, -Z_comp.imag, 'v', label=labels[2], **plotkwargs)
     if legend:
         plt.legend()
     if Z_fit is not None and np.all(omega == omega_fit) and compare:
         plot_compare_to_data(omega, Z, Z_fit, subplot=224, residual=residual, sign=sign,
-                             limits=limits_residual, legend=legend)
+                             limits=limits_residual, legend=legend, **plotkwargs)
     plt.tight_layout()
     if save and not append:
-        plt.savefig(str(title).replace(" ", "_") + "_impedance_overview.pdf")
+        cleantitle = str(title).replace(" ", "_").replace("/", "")
+        plt.savefig(cleantitle + "_impedance_overview.pdf")
     if show:
         plt.show()
     elif not show and not append:
@@ -651,7 +747,7 @@ def plot_impedance(omega, Z, title="", Z_fit=None, show=True, save=False, Z_comp
 
 def plot_compare_to_data(omega, Z, Z_fit, subplot=None, title="", show=True, save=False,
                          residual="parts", sign=False, limits=None, impedance_threshold=1.,
-                         legend=True):
+                         legend=True, **plotkwargs):
     '''
     plots the difference of the fitted function to the data
 
@@ -733,10 +829,10 @@ def plot_compare_to_data(omega, Z, Z_fit, subplot=None, title="", show=True, sav
         plt.title((str(title) + "relative difference to data").capitalize())
     plt.xlabel('Frequency / Hz')
     plt.ylabel(label)
-    plt.plot(omega / (2. * np.pi), diff_real, label='Real part', lw=3)
-    plt.plot(omega / (2. * np.pi), diff_imag, label='Imag part', lw=3, linestyle="--")
+    plt.plot(omega / (2. * np.pi), diff_real, label='Real part', **plotkwargs)
+    plt.plot(omega / (2. * np.pi), diff_imag, label='Imag part', linestyle="--", **plotkwargs)
     if residual != "absolute":
-        plt.plot(omega / (2. * np.pi), diff_abs, label='Abs value', linestyle="-.", lw=3)
+        plt.plot(omega / (2. * np.pi), diff_abs, label='Abs value', linestyle="-.", **plotkwargs)
     if limits is not None:
         assert len(limits) == 2, "You need to provide upper and lower limit!"
         plt.ylim(limits)
